const { Client, LocalAuth, MessageMedia, List, Location } = require('whatsapp-web.js');
const express = require('express');
const { body, validationResult } = require('express-validator');
const socketIO = require('socket.io');
const qrcode = require('qrcode');
const http = require('http');
const fileUpload = require('express-fileupload');
const axios = require('axios');
const mime = require('mime-types');
const port = process.env.PORT || 1000;
const app = express();
const server = http.createServer(app);
const io = socketIO(server);
const db = require('./helpers/mysql-corpobueno.js');

app.use(express.json());
app.use(express.urlencoded({
extended: true
}));
app.use(fileUpload({
debug: true
}));
app.use("/", express.static(__dirname + "/"))

app.get('/', (req, res) => {
  res.sendFile('index.html', {
    root: __dirname
  });
});

const client = new Client({
  authStrategy: new LocalAuth({ clientId: 'bot_corpobueno_1000' }),
  puppeteer: { headless: true,
    args: [
      '--no-sandbox',
      '--disable-setuid-sandbox',
      '--disable-dev-shm-usage',
      '--disable-accelerated-2d-canvas',
      '--no-first-run',
      '--no-zygote',
      '--single-process', // <- this one doesn't works in Windows
      '--disable-gpu'
    ] }
});

client.initialize();

io.on('connection', function(socket) {
  socket.emit('message', '© BOT-ZDG - Iniciado');
  socket.emit('qr', './icon.svg');

client.on('qr', (qr) => {
    console.log('QR RECEIVED', qr);
    qrcode.toDataURL(qr, (err, url) => {
      socket.emit('qr', url);
      socket.emit('message', '© BOT-ZDG QRCode recebido, aponte a câmera  seu celular!');
    });
});

client.on('ready', () => {
    socket.emit('ready', '© BOT-ZDG Dispositivo pronto!');
    socket.emit('message', '© BOT-ZDG Dispositivo pronto!');
    socket.emit('qr', './check.svg')	
    console.log('© BOT-ZDG Dispositivo pronto');
});

client.on('authenticated', () => {
    socket.emit('authenticated', '© BOT-ZDG Autenticado!');
    socket.emit('message', '© BOT-ZDG Autenticado!');
    console.log('© BOT-ZDG Autenticado');
});

client.on('auth_failure', function() {
    socket.emit('message', '© BOT-ZDG Falha na autenticação, reiniciando...');
    console.error('© BOT-ZDG Falha na autenticação');
});

client.on('change_state', state => {
  console.log('© BOT-ZDG Status de conexão: ', state );
});

client.on('disconnected', (reason) => {
  socket.emit('message', '© BOT-ZDG Cliente desconectado!');
  console.log('© BOT-ZDG Cliente desconectado', reason);
  client.initialize();
});
});

// Send message
app.post('/send-message', [
  body('number').notEmpty(),
  body('message').notEmpty(),
], async (req, res) => {
  const errors = validationResult(req).formatWith(({
    msg
  }) => {
    return msg;
  });

  if (!errors.isEmpty()) {
    return res.status(422).json({
      status: false,
      message: errors.mapped()
    });
  }

  const number = req.body.number;
  const numberDDD = number.substr(0, 2);
  const numberUser = number.substr(-8, 8);
  const message = req.body.message;

  if (numberDDD <= 30) {
    const numberZDG = "55" + numberDDD + "9" + numberUser + "@c.us";
    client.sendMessage(numberZDG, message).then(response => {
    res.status(200).json({
      status: true,
      message: 'BOT-ZDG Mensagem enviada',
      response: response
    });
    }).catch(err => {
    res.status(500).json({
      status: false,
      message: 'BOT-ZDG Mensagem não enviada',
      response: err.text
    });
    });
  }
  else if (numberDDD > 30) {
    const numberZDG = "55" + numberDDD + numberUser + "@c.us";
    client.sendMessage(numberZDG, message).then(response => {
    res.status(200).json({
      status: true,
      message: 'BOT-ZDG Mensagem enviada',
      response: response
    });
    }).catch(err => {
    res.status(500).json({
      status: false,
      message: 'BOT-ZDG Mensagem não enviada',
      response: err.text
    });
    });
  }
});


// Send media
app.post('/send-media', async (req, res) => {
  const number = req.body.number;
  const numberDDD = number.substr(0, 2);
  const numberUser = number.substr(-8, 8);
  const caption = req.body.caption;
  const fileUrl = req.body.file;

  let mimetype;
  const attachment = await axios.get(fileUrl, {
    responseType: 'arraybuffer'
  }).then(response => {
    mimetype = response.headers['content-type'];
    return response.data.toString('base64');
  });

  const media = new MessageMedia(mimetype, attachment, 'Media');

  if (numberDDD <= 30) {
    const numberZDG = "55" + numberDDD + "9" + numberUser + "@c.us";
    client.sendMessage(numberZDG, media, {caption: caption}).then(response => {
    res.status(200).json({
      status: true,
      message: 'BOT-ZDG Imagem enviada',
      telefone: numberZDG,
      response: response
    });
    }).catch(err => {
    res.status(500).json({
      status: false,
      message: 'BOT-ZDG Imagem não enviada',
      response: err.text
    });
    });
  }

  else if (numberDDD > 30) {
    const numberZDG = "55" + numberDDD + numberUser + "@c.us";
    client.sendMessage(numberZDG, media, {caption: caption}).then(response => {
    res.status(200).json({
      status: true,
      message: 'BOT-ZDG Imagem enviada',
      telefone: numberZDG,
      response: response
    });
    }).catch(err => {
    res.status(500).json({
      status: false,
      message: 'BOT-ZDG Imagem não enviada',
      response: err.text
    });
    });
  }
});



const EventEmitter = require('eventemitter3');

class MyEmitter extends EventEmitter {}

const myEmitter = new MyEmitter();
require('eventemitter3').EventEmitter.prototype._maxListeners = 70;
require('eventemitter3').defaultMaxListeners = 70;

// increase the limit
//myEmitter.setMaxListeners(11);

for(let i = 0; i < 11; i++) {
  myEmitter.on('eventemitter3', _ => console.log(i));
}

myEmitter.emit('eventemitter3');


client.on('message', async msg => {
  console.log('Mensagem recebida', msg);
  
  const user = msg.from; //.replace(/\D/g, '')
  const cliente = msg.from.replace(/\D/g, '');
  const remetente = msg.to;
  const ack = msg.ack;
  const tipo = msg.type;
  const body = msg.body;
  const nomecliente = msg._data.notifyName;
  const getUserFrom = await db.getUser(user);
  //const keyword = msg.body.toLowerCase();
  const keyword = msg.body.replace(/\D/g, '');
  const replyMessage = await db.getReply(keyword);
  const getUserStatus = await db.getStatus(user);
  const getid_chat = await db.getid_chat(user);
  const atualizarcliente = await db.getatualiza(user);
//  console.log('id chat',getid_chat);
  console.log('bot', getUserStatus);
  console.log('chatid',user);
  console.log('nome',cliente);
  console.log('nome1',nomecliente);
  console.log('textorobo',replyMessage);
  
  if( msg.body == '' ){

    const body = msg._data.body;
    
  }else{

    const body = msg.body;
    
  }
  
  //const getid_chat = "22";
  

    if (getUserFrom === false) {
    //  const chave = await db.setUser(user);
     // await db.setMensagem(user,remetente,ack,body,tipo);

     const numeral = msg.body.replace(/\D/g, '');
     switch(numeral){
      case '99':
        
        client.sendMessage(msg.from, replyMessage);
        break;
        case '98':
        
        client.sendMessage(msg.from, replyMessage);
        break;
     
      case '1':
      await db.setUser(user,nomecliente,'17',remetente);
      client.sendMessage(msg.from, replyMessage);
      break;
      case '2':
    
          await db.setUser(user,nomecliente,'101',remetente);
          const caminho_base = await db.base('1000');
          const numbercliente = cliente.substr(-8, 8);
          
  
          await db.cancela_disparo(numbercliente,caminho_base);
          console.log('caminho',caminho_base);
          client.sendMessage(msg.from, replyMessage);
    
          break;
        
          case '8':
    
          await db.setUser(user,nomecliente,'28',remetente);
          client.sendMessage(msg.from, replyMessage);
    
          break;
        
          default:
         await db.setUser(user,nomecliente,'102',remetente);
      
          const replyMessageWelcome = await db.getReply('oi');
          const replytexto = await db.texto('1000');
          const replyimagem = await db.imagem('1000');
          const media = MessageMedia.fromFilePath(replyimagem);
          client.sendMessage(msg.from, media, {caption: replytexto});
         
         setTimeout(function() {
          client.sendMessage(msg.from, replyMessageWelcome);
        },10000 + Math.floor(Math.random() * 5000));
         
          
        
          console.log('imagem',replyimagem);
          console.log('campanha',replytexto);
          break;
      }
          
    }else if (atualizarcliente !== false){
      const numeral = msg.body.replace(/\D/g, '');
      
      switch(numeral){
        case '99':
          
          client.sendMessage(msg.from, replyMessage);
          console.log('atualizar',atualizarcliente);
          break;
          case '98':
        
        client.sendMessage(msg.from, replyMessage);
        break;
      case '1':
        await db.atendente(user,'17',remetente);
        client.sendMessage(msg.from, replyMessage);
        console.log('atualizar',atualizarcliente);
        break;
       case '8':
        await db.atendente(user,'28',remetente);
        client.sendMessage(msg.from, replyMessage);
        console.log('atualizar',atualizarcliente);
        console.log('valores',numeral);
        console.log('texto',replyMessage);
         break;

         case '2':
          const caminho_base = await db.base('1000');
          const numbercliente = cliente.substr(-8, 8);
          await db.atendente(user,'101',remetente);
  
          await db.cancela_disparo(numbercliente,caminho_base);
          console.log('caminho',caminho_base);
          client.sendMessage(msg.from, replyMessage);
           break;
          }

     } else if (msg.body === '10'){
          await db.setStatusOff(user);
          client.sendMessage(msg.from, 'ChatBot OFF');
    }
    else if (msg.body === '9'){
          await db.setStatusOn(user);
          const replyMessageWelcome = await db.getReply('oi');
          client.sendMessage(msg.from, replyMessageWelcome);
    }
    else if (replyMessage !== false && getUserStatus === 'on'){

      
     
     // const AtualizaCliente = await db.getatualiza(user);
   
     // await db.setMensagem(user,remetente,ack,body,tipo,chave);
      
     
      if (msg.body === '1'){
       await db.atendente(user,'17',remetente);
        

      }
      if (msg.body === '2'){
      //  const caminho_base= await db.base('1001');
      //  const numbercliente = cliente.substr(-8, 8);
        await db.atendente(user,'101',remetente);

        await db.cancela_disparo(cliente);
        

      }
      if (msg.body === '3'){
        await db.atendente(user,'3',remetente);
       

      }
      if (msg.body === '4'){
        await db.atendente(user,'101',remetente);
        await db.cancela_disparo(cliente);
       

      }
      
          client.sendMessage(msg.from, replyMessage);
          console.log('texto robo',replyMessage);


    }else if (getUserStatus === 'off'){
      client.sendMessage(msg.from, 'O humano vai te atender.');
      await db.setMensagem(user,remetente,ack,body,tipo);
  }
  await db.setMensagem(user,remetente,ack,body,tipo);
  
});

    
server.listen(port, function() {
        console.log('App running on *: ' + port);
});
